#region Using

using System;
using System.IO;
using System.Collections.Generic;
using System.Globalization;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

#endregion

namespace FolderCleaner
{
  public partial class FormMain : Form
  {
    public FormMain()
    {
      InitializeComponent();
      cbPredefinedFileTypes.DropDownStyle = ComboBoxStyle.DropDownList;
      cbPredefinedFileTypes.SelectedIndex = 0;
    }

    private const string DELETED_FILENAME = "!_Deleted files";
    private System.Collections.Specialized.StringCollection _FileTypes;

    #region Events

    private void btnBrowse_Click(object sender, EventArgs e)
    {
      if (folderBrowserDialog1.ShowDialog() == DialogResult.OK)
      {
        txtFolder.Text = folderBrowserDialog1.SelectedPath;
        btnFind.Enabled = IsValid();
      }
    }

    private void btnFind_Click(object sender, EventArgs e)
    {
      cblFolders.Items.Clear();
      FindEmptyFolders();
      btnDelete.Enabled = true;
      UpdateSelectionCount();
    }

    private void rbDelete1_CheckedChanged(object sender, EventArgs e)
    {
      txtFileTypes.Enabled = false;
      cbPredefinedFileTypes.Enabled = false;
      btnFind.Enabled = IsValid();
      btnDelete.Enabled = IsValid();
    }

    private void rbDelete2_CheckedChanged(object sender, EventArgs e)
    {
      txtFileTypes.Enabled = true;
      cbPredefinedFileTypes.Enabled = true;
      btnFind.Enabled = IsValid();
      btnDelete.Enabled = IsValid();
    }

    private void rbDelete3_CheckedChanged(object sender, EventArgs e)
    {
      txtFileTypes.Enabled = true;
      cbPredefinedFileTypes.Enabled = true;
      btnFind.Enabled = IsValid();
      btnDelete.Enabled = IsValid();
    }

    private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
    {
      switch (cbPredefinedFileTypes.SelectedIndex)
      {
        case 1: // Audio files
          txtFileTypes.Text = Properties.Resources.FileTypesAudio;
          break;
        case 2: // Video files
          txtFileTypes.Text = Properties.Resources.FileTypesVideo;
          break;
        case 3: // HTML files
          txtFileTypes.Text = Properties.Resources.FileTypesHtml;
          break;
        case 4: // Images
          txtFileTypes.Text = Properties.Resources.FileTypesImages;
          break;
      }
    }

    private void txtFileTypes_TextChanged(object sender, EventArgs e)
    {
      foreach (char c in txtFileTypes.Text.ToCharArray())
      {
        if (char.IsLetterOrDigit(c) || c == ',' || char.IsWhiteSpace(c))
        {
          lblError.Text = string.Empty;
        }
        else
        {
          lblError.Text = Properties.Resources.ValidationMessage;
        }
      }

      btnFind.Enabled = IsValid();
      btnDelete.Enabled = IsValid();
    }

    private void cblFolders_SelectedIndexChanged(object sender, EventArgs e)
    {
      UpdateSelectionCount();
    }

    private void btnDelete_Click(object sender, EventArgs e)
    {
      if (MessageBox.Show(Properties.Resources.AreYouSureYouWantToDelete, this.Text, MessageBoxButtons.YesNo) == DialogResult.Yes)
      {
        string dir = txtFolder.Text + "\\" + DELETED_FILENAME + " " + DateTime.Now.ToString("yyyy-MM-dd HH-mm-ss", CultureInfo.InvariantCulture);
        DirectoryInfo copyDir = Directory.CreateDirectory(dir);
        PrepareProgressBar(cblFolders.CheckedItems.Count);

        foreach (string folder in cblFolders.CheckedItems)
        {
          if (Directory.Exists(folder))
            Directory.Move(folder, copyDir.FullName + "\\" + new DirectoryInfo(folder).Name);

          pbLoadFolders.Increment(1);
        }

        pbLoadFolders.Visible = false;
        MessageBox.Show(Properties.Resources.EmptyFoldersAreMovedTo + "\n" + copyDir.FullName);
      }
    }

    #endregion

    #region Helper methods

    /// <summary>
    /// Checks if the form is valid, based on the 
    /// choises made by the user.
    /// </summary>
    private bool IsValid()
    {
      if (!string.IsNullOrEmpty(txtFolder.Text))
      {
        if (rbDelete1.Checked)
          return true;

        if (rbDelete2.Checked || rbDelete3.Checked)
        {
          if (string.IsNullOrEmpty(lblError.Text) && txtFileTypes.Text.Length > 0)
            return true;
        }
      }

      return false;
    }

    /// <summary>
    /// Adds the specified file types to the collection
    /// </summary>
    private void RegisterSelectedFileTypes()
    {
      if (_FileTypes == null)
        _FileTypes = new System.Collections.Specialized.StringCollection();

      string[] fileTypes = txtFileTypes.Text.Split(',');
      _FileTypes.Clear();

      foreach (string s in fileTypes)
      {
        if (!s.StartsWith("."))
          _FileTypes.Add("." + s.Trim().ToLowerInvariant());
        else
          _FileTypes.Add(s.Trim().ToLowerInvariant());
      }
    }

    /// <summary>
    /// Checks if a directory or file is system-, hidden- or readonly.
    /// </summary>
    private static bool IsAttributesAllowed(FileAttributes attributes)
    {
      return (attributes & (FileAttributes.Hidden | FileAttributes.System | FileAttributes.ReadOnly)) == 0;
    }

    private void UpdateSelectionCount()
    {
      lbSelectedFiles.Text = cblFolders.CheckedItems.Count + " " + Properties.Resources.SelectedFolders;
    }

    /// <summary>
    /// Resets the progress bar and setting its max value
    /// </summary>
    private void PrepareProgressBar(int maximum)
    {
      pbLoadFolders.Value = 0;
      pbLoadFolders.Maximum = maximum;
      pbLoadFolders.Visible = true;
    }

    #endregion

    #region Find folders

    private void FindEmptyFolders()
    {
      if (!rbDelete1.Checked)
        RegisterSelectedFileTypes();

      DirectoryInfo[] directories = new DirectoryInfo(txtFolder.Text).GetDirectories();
      PrepareProgressBar(directories.Length);
      cblFolders.BeginUpdate();

      foreach (DirectoryInfo dir in directories)
      {
        if (IsAttributesAllowed(dir.Attributes) && IsChildFoldersEmpty(dir) && IsFolderEmpty(dir))
        {
          cblFolders.Items.Add(dir.FullName, true);
        }

        pbLoadFolders.Increment(1);
      }

      cblFolders.Sorted = true;
      cblFolders.EndUpdate();
      pbLoadFolders.Visible = false;
    }

    private bool IsChildFoldersEmpty(DirectoryInfo dir)
    {
      bool isEmpty = true;

      DirectoryInfo[] directories = dir.GetDirectories();

      foreach (DirectoryInfo subDir in directories)
      {
        if (IsAttributesAllowed(subDir.Attributes))
        {
          if (IsChildFoldersEmpty(subDir))
          {
            if (IsFolderEmpty(subDir))
            {
              cblFolders.Items.Add(subDir.FullName, true);
              isEmpty = true;
            }
            else
            {
              isEmpty = false;
            }
          }
        }
        else
        {
          isEmpty = false;
        }
      }

      return isEmpty;
    }

    private bool IsFolderEmpty(DirectoryInfo dir)
    {
      string[] files = Directory.GetFiles(dir.FullName);
      if (files.Length > 0)
      {
        foreach (string filename in files)
        {
          FileInfo file = new FileInfo(filename);
          if (_FileTypes != null && IsAttributesAllowed(file.Attributes))
          {            
            if (rbDelete2.Checked) // where extenstions are present
            {
              if (_FileTypes.Contains(file.Extension.ToLowerInvariant()))
                return true;
            }
            else if (rbDelete3.Checked) // where extenstions are NOT present
            {
              if (_FileTypes.Contains(file.Extension.ToLowerInvariant()))
                return false;
            }
          }
        }

        if (rbDelete2.Checked) // where extenstions are present
          return false;
        else if (rbDelete3.Checked) // where extenstions are NOT present
          return true;
        else
          return false;
      }

      if (rbDelete2.Checked) // where extenstions are present
        return false;
      else if (rbDelete3.Checked) // where extenstions are NOT present
        return true;
      else
        return true;
    }

    #endregion

  }
}